#ifndef _KAAPI_SCHED_H_
#define _KAAPI_SCHED_H_

typedef struct kaapi_sched_t {
  void                (*init)           (void);
  void                (*push)           (kaapi_tasklist_t*	const, kaapi_taskdescr_t* const);
  uint32_t            (*push_activated) (kaapi_tasklist_t*	const, kaapi_taskdescr_t* const);
  kaapi_taskdescr_t*  (*steal)          (kaapi_processor_t* const, kaapi_taskdescr_t* const);
  void                (*prologue)       (kaapi_taskdescr_t* const);
  void                (*epilogue)       (kaapi_taskdescr_t* const);
  void                (*finalize)       (void);
} kaapi_sched_t;

extern kaapi_sched_t kaapi_scheduler;

extern void kaapi_sched_init(void);

extern void kaapi_sched_finalize(void);

static inline uint32_t kaapi_sched_push_activated( kaapi_tasklist_t*	const tasklist,
                                      kaapi_taskdescr_t* const td )
{
  return kaapi_scheduler.push_activated(tasklist, td);
}

static inline void kaapi_sched_push( kaapi_tasklist_t*	const tasklist,
                                kaapi_taskdescr_t* const td )
{
  kaapi_scheduler.push(tasklist, td);
}

static inline kaapi_taskdescr_t* kaapi_sched_steal(kaapi_processor_t* const thief, kaapi_taskdescr_t* const td)
{
  return kaapi_scheduler.steal(thief, td);
}

static inline void kaapi_sched_task_epilogue(kaapi_taskdescr_t* const td)
{
  kaapi_scheduler.epilogue(td);
}

static inline void kaapi_sched_task_prologue(kaapi_taskdescr_t* const td)
{
  kaapi_scheduler.prologue(td);
}

static inline int kaapi_sched_push_by_site(kaapi_taskdescr_t* const td)
{
  /* always take the first bit ==1 */
  int kid = kaapi_taskdescr_get_site(td)-1;
  kaapi_assert_debug( (kid >=0) && (kid < (int32_t)kaapi_count_kprocessors) );
  kaapi_processor_t* kproc_remote = kaapi_all_kprocessors[kid];
  /* forbid steal of this task */
  kaapi_task_set_unstealable(td->task);
  return kaapi_readylist_remote_push( kproc_remote->rtl, td );
}

/** Used to steal a tasklist in the frame
*/
extern void kaapi_sched_stealtasklist( 
                           kaapi_thread_context_t*       thread, 
                           kaapi_tasklist_t*             tasklist, 
                           kaapi_listrequest_t*          lrequests, 
                           kaapi_listrequest_iterator_t* lrrange
);

/** Used to steal a tasklist from local readylist of victim
*/
extern void kaapi_sched_stealreadytasklist( 
                           kaapi_thread_context_t*       thread, 
                           kaapi_processor_t*            proc, 
                           kaapi_listrequest_t*          lrequests, 
                           kaapi_listrequest_iterator_t* lrrange
);

extern void kaapi_sched_stealreadytasklist_priority( 
                           kaapi_thread_context_t*       thread, 
                           kaapi_processor_t*            kproc,
                           kaapi_listrequest_t*          lrequests, 
                           kaapi_listrequest_iterator_t* lrrange
);

/* try to pop a task in priority queue of the current kproc (assumed to be idle in sched_idle
   or sched_suspend).
   The task is pushed in the thread top frame and execution must be done using kaapi_thread_execframe 
   function.
*/
extern int kaapi_sched_popreadytasklist_priority(
                           kaapi_thread_context_t*       thread, 
                           kaapi_processor_t*            kproc
);


/** Start execution with a TD.
    Initialize the stack of the current thread and then, the caller shoud calls kaapi_thread_execframe_tasklist.
    Return 0 in case of success.
*/
extern int kaapi_thread_startexecwithtd( 
      kaapi_processor_t* kproc,
      kaapi_taskdescr_t* td
);

int kaapi_thread_execframe_tasklist( kaapi_thread_context_t* thread );

#endif /* _KAAPI_SCHED_H_ */
