
#include "kaapi_impl.h"

#include "kaapi_sched.h"
#include "ws/kaapi_sched_ws.h"
#include "locality_writer/kaapi_sched_locality_writer.h"

#if defined(KAAPI_USE_PERFMODEL)
#include "heft/kaapi_sched_heft.h"
#include "gdual/kaapi_sched_gdual.h"
#include "speedup/kaapi_sched_speedup.h"
#include "daws/kaapi_sched_daws.h"
#endif /* KAAPI_USE_PERFMODEL */

kaapi_sched_t kaapi_scheduler;

static kaapi_sched_t kaapi_scheduler_ws = {
  .init             = kaapi_sched_ws_init,
  .finalize         = kaapi_sched_ws_finalize,
  .push             = kaapi_sched_ws_push,
  .push_activated   = kaapi_sched_ws_push_activated,
  .steal            = kaapi_sched_ws_steal,
  .prologue         = kaapi_sched_ws_task_prologue,
  .epilogue         = kaapi_sched_ws_task_epilogue
};

static kaapi_sched_t kaapi_scheduler_affinity_writer = {
  .init             = kaapi_sched_locality_writer_init,
  .finalize         = kaapi_sched_locality_writer_finalize,
  .push             = kaapi_sched_locality_writer_push,
  .push_activated   = kaapi_sched_locality_writer_push_activated,
  .steal            = kaapi_sched_ws_steal,
  .prologue         = kaapi_sched_locality_writer_task_prologue,
  .epilogue         = kaapi_sched_locality_writer_task_epilogue
};

#if defined(KAAPI_USE_PERFMODEL)
static kaapi_sched_t kaapi_scheduler_heft = {
  .init             = kaapi_sched_heft_init,
  .finalize         = kaapi_sched_heft_finalize,
  .push             = kaapi_sched_heft_push,
  .push_activated   = kaapi_sched_heft_push_activated,
  .steal            = kaapi_sched_heft_steal,
  .prologue         = kaapi_sched_heft_task_prologue,
  .epilogue         = kaapi_sched_heft_task_epilogue
};

static kaapi_sched_t kaapi_scheduler_gdual = {
  .init             = kaapi_sched_gdual_init,
  .finalize         = kaapi_sched_gdual_finalize,
  .push             = kaapi_sched_gdual_push,
  .push_activated   = kaapi_sched_gdual_push_activated,
  .steal            = kaapi_sched_gdual_steal,
  .prologue         = kaapi_sched_gdual_task_prologue,
  .epilogue         = kaapi_sched_gdual_task_epilogue
};

static kaapi_sched_t kaapi_scheduler_speedup = {
  .init             = kaapi_sched_speedup_init,
  .finalize         = kaapi_sched_speedup_finalize,
  .push             = kaapi_sched_speedup_push,
  .push_activated   = kaapi_sched_ws_push_activated,
  .steal            = kaapi_sched_speedup_steal,
  .prologue         = kaapi_sched_speedup_task_prologue,
  .epilogue         = kaapi_sched_speedup_task_epilogue
};

static kaapi_sched_t kaapi_scheduler_daws = {
  .init             = kaapi_sched_daws_init,
  .finalize         = kaapi_sched_daws_finalize,
  .push             = kaapi_sched_daws_push,
  .push_activated   = kaapi_sched_daws_push_activated,
  .steal            = kaapi_sched_daws_steal,
  .prologue         = kaapi_sched_daws_task_prologue,
  .epilogue         = kaapi_sched_daws_task_epilogue
};
#endif

void kaapi_sched_init(void)
{
  const char* sched;

  sched = getenv("KAAPI_SCHED");
  kaapi_scheduler = kaapi_scheduler_ws;
  if( sched != 0 )
  {
    if (strcmp(sched, "writer") == 0)
      kaapi_scheduler = kaapi_scheduler_affinity_writer;
#if defined(KAAPI_USE_PERFMODEL)
    else if (strcmp(sched, "heft") == 0)
      kaapi_scheduler = kaapi_scheduler_heft;
    else if (strcmp(sched, "gdual") == 0)
      kaapi_scheduler = kaapi_scheduler_gdual;
    else if (strcmp(sched, "speedup") == 0)
      kaapi_scheduler = kaapi_scheduler_speedup;
    else if (strcmp(sched, "daws") == 0)
      kaapi_scheduler = kaapi_scheduler_daws;
#endif
    else if (strcmp(sched, "default") == 0)
      kaapi_scheduler = kaapi_scheduler_ws;
    else {
      fprintf(stderr, "*** Kaapi: bad value for 'KAAPI_SCHED': '%s'\n",
              getenv("KAAPI_SCHED")
              );
    }
  }
  
  kaapi_scheduler.init();
}

void kaapi_sched_finalize(void)
{
  kaapi_scheduler.finalize();
}
